<?php
/**
 * Handler: for managing Wordpress pages.
 */
namespace EwTools\Handler;

use EwTools\Support\Str;
use WP_Post;

/**
 * Pages
 * @todo Make ORM-object for Pages of this class
 *
 * @link    http://infomaker.se
 * @package EwTools\Handler
 * @since   EwTools\Handler\Pages 0.1
 */
class Pages {
	
	/**
	 * Defaults for fetching pages in Wordpress
	 *
	 * @var array
	 */
	protected static $default_options = [
		'sort_order' => 'ASC',
		'sort_column' => 'post_title',
		'hierarchical' => true,
		'exclude' => '',
		'include' => '',
		'meta_key' => '',
		'meta_value' => '',
		'authors' => '',
		'child_of' => 0,
		'parent' => -1,
		'exclude_tree' => '',
		'number' => '',
		'offset' => 0,
		'post_type' => 'page',
		'post_status' => 'publish'
	];
	
	/**
	 * Retrieve all pages with optional options for filtering
	 *
	 * @param array $options
	 *
     * @uses get_pages()
     *
	 * @since 0.1
	 * @return array
	 */
	public static function get( $options=[] ) {
		return get_pages(array_replace_recursive(static::$default_options, $options));
	}
	
	/**
	 * Use page title to retrieve the pages content as string of html
	 *
	 * @param string $title
     *
     * @uses WP_Post
	 *
	 * @since 0.1
	 * @return string
	 */
	public static function scrapeByTitle( $title ) {
		$page = static::getByTitle($title);
		if ( $page instanceof WP_Post) {
			return static::getContentHtml($page);
		}
		return '';
	}
	
	/**
	 * Retrieve the pages content as string of html
	 *
	 * @param WP_Post $page
	 *
     * @uses get_permalink()
     * @uses wp_remote_get()
     * @uses is_wp_error()
     * @uses wp_remote_retrieve_body()
     *
	 * @since 0.1
	 * @return string
	 */
	public static function getContentHtml( WP_Post $page ) {
		
		$url = get_permalink( $page->ID );
		$response = wp_remote_get($url);
		
		// Check for error
		if (! is_wp_error( $response ) ) {
			$content = wp_remote_retrieve_body($response);
			// Check for error
			if (! is_wp_error( $content ) ) {
				return $content;
			}
		}
		return '';
	}
	
	/**
	 * Retrieve page by title
	 * @param string $title
	 *
     * @uses get_page_by_title()
     *
	 * @since 0.1
	 * @return array|void|WP_Post
	 */
	public static function getByTitle( $title ) {
		return get_page_by_title($title);
	}
	
	/**
	 * Retrieve page by id
	 *
	 * @param string $id
	 *
     * @uses get_post()
     *
	 * @since 0.1
	 * @return array|void|WP_Post
	 */
	public static function getById( $id) {
		return get_post($id);
	}
	
	/**
	 * Retrieve page by url
	 *
	 * @param string $url
	 *
     * @uses url_to_postid()
     *
     * @since 0.1
	 * @return array|void|WP_Post
	 */
	public static function getByUrl( $url) {
		return static::getById(url_to_postid($url));
	}
	
	/**
	 * Retrieve the post meta from a page
	 *
	 * @param string $title
	 *
     * @uses get_post_meta()
     *
	 * @since 0.1
	 * @return array|mixed
	 */
	public static function getMetaFromTitle( $title ) {
		if ( ($page = static::getByTitle($title)) !== null ) {
			return get_post_meta($page->ID);
		}
		return [];
	}
	
	/**
	 * Retrieve an array of page links with "permalink" and "title".
	 * Add options to filter the resulted list
	 *
	 * @param array $options
	 * @param bool  $use_hierarchy
	 *
     * @uses get_permalink()
     *
	 * @since 0.1
	 * @return array
	 */
	public static function getLinks( $options=[], $use_hierarchy=true) {
		$options['hierarchical'] = $use_hierarchy;
		return array_map(function($page) use ($use_hierarchy) {
			return [
				'permalink' => get_permalink( $page->ID ),
				'title' => $use_hierarchy ? static::getHierarchicalText($page) : $page->post_title
			];
		}, static::get($options));
	}
	
	/**
	 * Retrieve an array of page data for use in select-box "value" and "text" and "id".
	 * Add options to filter the resulted list
	 *
	 * @param array $options
	 * @param bool  $use_hierarchy
	 *
     * @uses get_permalink()
     *
	 * @since 0.1
	 * @return array
	 */
	public static function getSelectData($options=[], $use_hierarchy=true) {
		$options['hierarchical'] = $use_hierarchy;
		$pages = static::get($options);
		return array_map(function($page) use ($use_hierarchy) {
			return [
				'text' =>  $use_hierarchy ? static::getHierarchicalText($page) : $page->post_title,
				'value' => $page->ID
			];
		}, $pages );
	}
	
	/**
	 * Add chars before the text to emphasise the posts hierarchical position to its ancestors.
	 *
	 * @param string $page
	 *
     * @uses get_post_ancestors()
     *
	 * @since 0.1
	 * @return string
	 */
	public static function getHierarchicalText( $page ) {
		$text = $page->post_title;
		$ancestors = $page->post_parent > 0 ? count(get_post_ancestors( $page )) : 0;
		return Str::prepend(str_repeat('— ', $ancestors), $text);
	}
}